# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'main_gui.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1098, 824)
        MainWindow.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.frame = QtWidgets.QFrame(self.centralwidget)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayout_2.addWidget(self.frame)
        self.tool_line = QtWidgets.QHBoxLayout()
        self.tool_line.setObjectName("tool_line")
        self.control_area = QtWidgets.QFrame(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.control_area.sizePolicy().hasHeightForWidth())
        self.control_area.setSizePolicy(sizePolicy)
        self.control_area.setFrameShape(QtWidgets.QFrame.Box)
        self.control_area.setFrameShadow(QtWidgets.QFrame.Raised)
        self.control_area.setObjectName("control_area")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.control_area)
        self.horizontalLayout_3.setContentsMargins(-1, -1, 9, -1)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.comboBox_back = QtWidgets.QComboBox(self.control_area)
        self.comboBox_back.setMinimumSize(QtCore.QSize(165, 0))
        self.comboBox_back.setEditable(False)
        self.comboBox_back.setCurrentText("")
        self.comboBox_back.setMaxVisibleItems(14)
        self.comboBox_back.setObjectName("comboBox_back")
        self.horizontalLayout_3.addWidget(self.comboBox_back)
        self.pushButton_pause = QtWidgets.QPushButton(self.control_area)
        self.pushButton_pause.setObjectName("pushButton_pause")
        self.horizontalLayout_3.addWidget(self.pushButton_pause)
        self.pushButton_start = QtWidgets.QPushButton(self.control_area)
        icon = QtGui.QIcon.fromTheme("SP_MediaPlay")
        self.pushButton_start.setIcon(icon)
        self.pushButton_start.setObjectName("pushButton_start")
        self.horizontalLayout_3.addWidget(self.pushButton_start)
        self.pushButton_next_job = QtWidgets.QPushButton(self.control_area)
        self.pushButton_next_job.setObjectName("pushButton_next_job")
        self.horizontalLayout_3.addWidget(self.pushButton_next_job)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.box_automatic = QtWidgets.QCheckBox(self.control_area)
        self.box_automatic.setObjectName("box_automatic")
        self.horizontalLayout_3.addWidget(self.box_automatic)
        self.tool_line.addWidget(self.control_area)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.tool_line.addItem(spacerItem1)
        self.label_current_progress = QtWidgets.QLabel(self.centralwidget)
        self.label_current_progress.setMinimumSize(QtCore.QSize(130, 0))
        self.label_current_progress.setText("")
        self.label_current_progress.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_current_progress.setObjectName("label_current_progress")
        self.tool_line.addWidget(self.label_current_progress)
        self.progressBar_current = QtWidgets.QProgressBar(self.centralwidget)
        self.progressBar_current.setProperty("value", 0)
        self.progressBar_current.setObjectName("progressBar_current")
        self.tool_line.addWidget(self.progressBar_current)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.tool_line.addItem(spacerItem2)
        self.label_batch_progress = QtWidgets.QLabel(self.centralwidget)
        self.label_batch_progress.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_batch_progress.setObjectName("label_batch_progress")
        self.tool_line.addWidget(self.label_batch_progress)
        self.progressBar_batch = QtWidgets.QProgressBar(self.centralwidget)
        self.progressBar_batch.setProperty("value", 0)
        self.progressBar_batch.setObjectName("progressBar_batch")
        self.tool_line.addWidget(self.progressBar_batch)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.tool_line.addItem(spacerItem3)
        self.pushButton_quit = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_quit.setObjectName("pushButton_quit")
        self.tool_line.addWidget(self.pushButton_quit)
        self.tool_line.setStretch(1, 3)
        self.tool_line.setStretch(2, 5)
        self.tool_line.setStretch(3, 12)
        self.tool_line.setStretch(4, 3)
        self.tool_line.setStretch(5, 3)
        self.tool_line.setStretch(6, 12)
        self.tool_line.setStretch(7, 1)
        self.verticalLayout_2.addLayout(self.tool_line)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1098, 21))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuEdit = QtWidgets.QMenu(self.menubar)
        self.menuEdit.setObjectName("menuEdit")
        self.menuCalibrate = QtWidgets.QMenu(self.menubar)
        self.menuCalibrate.setObjectName("menuCalibrate")
        self.menuAbout_PSS = QtWidgets.QMenu(self.menubar)
        self.menuAbout_PSS.setObjectName("menuAbout_PSS")
        MainWindow.setMenuBar(self.menubar)
        self.statusBar = QtWidgets.QStatusBar(MainWindow)
        self.statusBar.setObjectName("statusBar")
        MainWindow.setStatusBar(self.statusBar)
        self.actionLoad_video_directory = QtWidgets.QAction(MainWindow)
        self.actionLoad_video_directory.setObjectName("actionLoad_video_directory")
        self.actionLoad_config = QtWidgets.QAction(MainWindow)
        self.actionLoad_config.setObjectName("actionLoad_config")
        self.actionSave_config = QtWidgets.QAction(MainWindow)
        self.actionSave_config.setObjectName("actionSave_config")
        self.actionLoad_postproc_config = QtWidgets.QAction(MainWindow)
        self.actionLoad_postproc_config.setObjectName("actionLoad_postproc_config")
        self.actionSave_postproc_config = QtWidgets.QAction(MainWindow)
        self.actionSave_postproc_config.setObjectName("actionSave_postproc_config")
        self.actionSave = QtWidgets.QAction(MainWindow)
        self.actionSave.setObjectName("actionSave")
        self.actionSave_as = QtWidgets.QAction(MainWindow)
        self.actionSave_as.setObjectName("actionSave_as")
        self.actionQuit = QtWidgets.QAction(MainWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionEdit_configuration = QtWidgets.QAction(MainWindow)
        self.actionEdit_configuration.setObjectName("actionEdit_configuration")
        self.actionEdit_postproc_config = QtWidgets.QAction(MainWindow)
        self.actionEdit_postproc_config.setObjectName("actionEdit_postproc_config")
        self.actionDe_activate_master_frames = QtWidgets.QAction(MainWindow)
        self.actionDe_activate_master_frames.setObjectName("actionDe_activate_master_frames")
        self.actionLoad_master_dark_frame = QtWidgets.QAction(MainWindow)
        self.actionLoad_master_dark_frame.setObjectName("actionLoad_master_dark_frame")
        self.actionLoad_master_flat_frame = QtWidgets.QAction(MainWindow)
        self.actionLoad_master_flat_frame.setObjectName("actionLoad_master_flat_frame")
        self.actionCreate_new_master_dark_frame = QtWidgets.QAction(MainWindow)
        self.actionCreate_new_master_dark_frame.setObjectName("actionCreate_new_master_dark_frame")
        self.actionCreate_new_master_flat_frame = QtWidgets.QAction(MainWindow)
        self.actionCreate_new_master_flat_frame.setObjectName("actionCreate_new_master_flat_frame")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionShow_Quickstart = QtWidgets.QAction(MainWindow)
        self.actionShow_Quickstart.setCheckable(True)
        self.actionShow_Quickstart.setChecked(True)
        self.actionShow_Quickstart.setObjectName("actionShow_Quickstart")
        self.menuFile.addAction(self.actionLoad_video_directory)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionSave)
        self.menuFile.addAction(self.actionSave_as)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionLoad_config)
        self.menuFile.addAction(self.actionSave_config)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menuEdit.addAction(self.actionEdit_configuration)
        self.menuCalibrate.addAction(self.actionDe_activate_master_frames)
        self.menuCalibrate.addSeparator()
        self.menuCalibrate.addAction(self.actionLoad_master_dark_frame)
        self.menuCalibrate.addAction(self.actionLoad_master_flat_frame)
        self.menuCalibrate.addSeparator()
        self.menuCalibrate.addAction(self.actionCreate_new_master_dark_frame)
        self.menuCalibrate.addAction(self.actionCreate_new_master_flat_frame)
        self.menuAbout_PSS.addAction(self.actionAbout)
        self.menuAbout_PSS.addAction(self.actionShow_Quickstart)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuEdit.menuAction())
        self.menubar.addAction(self.menuCalibrate.menuAction())
        self.menubar.addAction(self.menuAbout_PSS.menuAction())

        self.retranslateUi(MainWindow)
        self.comboBox_back.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Planetary System Stacker"))
        self.comboBox_back.setToolTip(_translate("MainWindow", "Repeat starting from an earlier processing phase, or go back to the previous job."))
        self.pushButton_pause.setToolTip(_translate("MainWindow", "Suspend job execution after the current step"))
        self.pushButton_pause.setText(_translate("MainWindow", "Pause"))
        self.pushButton_start.setToolTip(_translate("MainWindow", "Start or continue job execution."))
        self.pushButton_start.setText(_translate("MainWindow", "Start / Cont."))
        self.pushButton_next_job.setToolTip(_translate("MainWindow", "Skip the remaining phases of the current job and start with the next one."))
        self.pushButton_next_job.setText(_translate("MainWindow", "Next Job"))
        self.box_automatic.setToolTip(_translate("MainWindow", "Continue in automatic (batch) mode."))
        self.box_automatic.setText(_translate("MainWindow", "Automatic"))
        self.label_batch_progress.setText(_translate("MainWindow", "Batch progress"))
        self.pushButton_quit.setToolTip(_translate("MainWindow", "Exit the program. The current configuration will be saved for the next run."))
        self.pushButton_quit.setText(_translate("MainWindow", "Quit"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuEdit.setTitle(_translate("MainWindow", "Edit"))
        self.menuCalibrate.setTitle(_translate("MainWindow", "Calibrate"))
        self.menuAbout_PSS.setTitle(_translate("MainWindow", "Info"))
        self.actionLoad_video_directory.setText(_translate("MainWindow", "Open"))
        self.actionLoad_config.setText(_translate("MainWindow", "Load configuration"))
        self.actionLoad_config.setToolTip(_translate("MainWindow", "Load configuration from \".pss\" file"))
        self.actionSave_config.setText(_translate("MainWindow", "Save configuration"))
        self.actionSave_config.setToolTip(_translate("MainWindow", "Save configuration to \".pss\" file"))
        self.actionLoad_postproc_config.setText(_translate("MainWindow", "Load postproc config"))
        self.actionSave_postproc_config.setText(_translate("MainWindow", "Save postproc config"))
        self.actionSave.setText(_translate("MainWindow", "Save"))
        self.actionSave_as.setText(_translate("MainWindow", "Save as"))
        self.actionQuit.setText(_translate("MainWindow", "Quit"))
        self.actionEdit_configuration.setText(_translate("MainWindow", "Edit configuration"))
        self.actionEdit_postproc_config.setText(_translate("MainWindow", "Edit postproc config"))
        self.actionDe_activate_master_frames.setText(_translate("MainWindow", "De-activate master frames"))
        self.actionLoad_master_dark_frame.setText(_translate("MainWindow", "Load master dark frame"))
        self.actionLoad_master_flat_frame.setText(_translate("MainWindow", "Load master flat frame"))
        self.actionCreate_new_master_dark_frame.setText(_translate("MainWindow", "Create new master dark frame"))
        self.actionCreate_new_master_flat_frame.setText(_translate("MainWindow", "Create new master flat frame"))
        self.actionAbout.setText(_translate("MainWindow", "About PSS"))
        self.actionShow_Quickstart.setText(_translate("MainWindow", "Show quickstart"))
