# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'postproc_editor_gui.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_postproc_editor(object):
    def setupUi(self, postproc_editor):
        postproc_editor.setObjectName("postproc_editor")
        postproc_editor.resize(865, 758)
        postproc_editor.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        postproc_editor.setFrameShape(QtWidgets.QFrame.Panel)
        postproc_editor.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.gridLayout = QtWidgets.QGridLayout(postproc_editor)
        self.gridLayout.setObjectName("gridLayout")
        self.VersionManagerWidget = QtWidgets.QLabel(postproc_editor)
        self.VersionManagerWidget.setText("")
        self.VersionManagerWidget.setObjectName("VersionManagerWidget")
        self.gridLayout.addWidget(self.VersionManagerWidget, 1, 2, 1, 2)
        self.tabWidget_postproc_control = QtWidgets.QTabWidget(postproc_editor)
        self.tabWidget_postproc_control.setObjectName("tabWidget_postproc_control")
        self.tab_sharpening = QtWidgets.QWidget()
        self.tab_sharpening.setObjectName("tab_sharpening")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tab_sharpening)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.pushButton_add_layer = QtWidgets.QPushButton(self.tab_sharpening)
        self.pushButton_add_layer.setObjectName("pushButton_add_layer")
        self.verticalLayout_2.addWidget(self.pushButton_add_layer)
        self.scrollArea = QtWidgets.QScrollArea(self.tab_sharpening)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 400, 615))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout.setObjectName("verticalLayout")
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout_2.addWidget(self.scrollArea)
        self.tabWidget_postproc_control.addTab(self.tab_sharpening, "")
        self.tab_rgb = QtWidgets.QWidget()
        self.tab_rgb.setObjectName("tab_rgb")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tab_rgb)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.group_general = QtWidgets.QGroupBox(self.tab_rgb)
        self.group_general.setObjectName("group_general")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.group_general)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.checkBox_automatic = QtWidgets.QCheckBox(self.group_general)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkBox_automatic.sizePolicy().hasHeightForWidth())
        self.checkBox_automatic.setSizePolicy(sizePolicy)
        self.checkBox_automatic.setMinimumSize(QtCore.QSize(0, 0))
        self.checkBox_automatic.setObjectName("checkBox_automatic")
        self.verticalLayout_3.addWidget(self.checkBox_automatic)
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.fgw_label_parameter = QtWidgets.QLabel(self.group_general)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fgw_label_parameter.sizePolicy().hasHeightForWidth())
        self.fgw_label_parameter.setSizePolicy(sizePolicy)
        self.fgw_label_parameter.setMinimumSize(QtCore.QSize(0, 0))
        self.fgw_label_parameter.setObjectName("fgw_label_parameter")
        self.gridLayout_2.addWidget(self.fgw_label_parameter, 1, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(36, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem, 1, 4, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem1, 1, 6, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(40, 25, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem2, 1, 1, 1, 1)
        self.fgw_label_display = QtWidgets.QLabel(self.group_general)
        self.fgw_label_display.setObjectName("fgw_label_display")
        self.gridLayout_2.addWidget(self.fgw_label_display, 1, 5, 1, 1)
        self.label_resolution = QtWidgets.QLabel(self.group_general)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_resolution.sizePolicy().hasHeightForWidth())
        self.label_resolution.setSizePolicy(sizePolicy)
        self.label_resolution.setMinimumSize(QtCore.QSize(0, 0))
        self.label_resolution.setObjectName("label_resolution")
        self.gridLayout_2.addWidget(self.label_resolution, 0, 0, 1, 1)
        self.comboBox_resolution = QtWidgets.QComboBox(self.group_general)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBox_resolution.sizePolicy().hasHeightForWidth())
        self.comboBox_resolution.setSizePolicy(sizePolicy)
        self.comboBox_resolution.setMinimumSize(QtCore.QSize(90, 0))
        self.comboBox_resolution.setObjectName("comboBox_resolution")
        self.gridLayout_2.addWidget(self.comboBox_resolution, 0, 2, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem3, 0, 3, 1, 1)
        self.fgw_slider_value = QtWidgets.QSlider(self.group_general)
        self.fgw_slider_value.setMinimum(1)
        self.fgw_slider_value.setMaximum(6)
        self.fgw_slider_value.setSingleStep(1)
        self.fgw_slider_value.setPageStep(1)
        self.fgw_slider_value.setProperty("value", 4)
        self.fgw_slider_value.setSliderPosition(4)
        self.fgw_slider_value.setOrientation(QtCore.Qt.Horizontal)
        self.fgw_slider_value.setObjectName("fgw_slider_value")
        self.gridLayout_2.addWidget(self.fgw_slider_value, 1, 2, 1, 2)
        self.gridLayout_2.setColumnStretch(1, 1)
        self.gridLayout_2.setColumnStretch(2, 3)
        self.gridLayout_2.setColumnStretch(3, 8)
        self.gridLayout_2.setColumnStretch(4, 1)
        self.verticalLayout_3.addLayout(self.gridLayout_2)
        self.verticalLayout_3.setStretch(0, 1)
        self.verticalLayout_3.setStretch(1, 2)
        self.verticalLayout_4.addWidget(self.group_general)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem4)
        self.group_red = QtWidgets.QGroupBox(self.tab_rgb)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.group_red.setFont(font)
        self.group_red.setObjectName("group_red")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.group_red)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.pushButton_red_up = QtWidgets.QPushButton(self.group_red)
        self.pushButton_red_up.setStyleSheet("")
        self.pushButton_red_up.setObjectName("pushButton_red_up")
        self.gridLayout_3.addWidget(self.pushButton_red_up, 1, 2, 1, 1)
        self.label_red_left = QtWidgets.QLabel(self.group_red)
        self.label_red_left.setStyleSheet("QLabel { color :red; }")
        self.label_red_left.setAlignment(QtCore.Qt.AlignCenter)
        self.label_red_left.setObjectName("label_red_left")
        self.gridLayout_3.addWidget(self.label_red_left, 2, 0, 1, 1)
        self.pushButton_red_down = QtWidgets.QPushButton(self.group_red)
        self.pushButton_red_down.setStyleSheet("")
        self.pushButton_red_down.setObjectName("pushButton_red_down")
        self.gridLayout_3.addWidget(self.pushButton_red_down, 3, 2, 1, 1)
        self.label_red_down = QtWidgets.QLabel(self.group_red)
        self.label_red_down.setStyleSheet("QLabel { color :red; }")
        self.label_red_down.setAlignment(QtCore.Qt.AlignCenter)
        self.label_red_down.setObjectName("label_red_down")
        self.gridLayout_3.addWidget(self.label_red_down, 4, 2, 1, 1)
        self.pushButton_red_reset = QtWidgets.QPushButton(self.group_red)
        self.pushButton_red_reset.setObjectName("pushButton_red_reset")
        self.gridLayout_3.addWidget(self.pushButton_red_reset, 2, 2, 1, 1)
        self.label_red_right = QtWidgets.QLabel(self.group_red)
        self.label_red_right.setStyleSheet("QLabel { color :red; }")
        self.label_red_right.setAlignment(QtCore.Qt.AlignCenter)
        self.label_red_right.setObjectName("label_red_right")
        self.gridLayout_3.addWidget(self.label_red_right, 2, 4, 1, 1)
        self.label_red_up = QtWidgets.QLabel(self.group_red)
        self.label_red_up.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_red_up.setStyleSheet("QLabel { color :red; }")
        self.label_red_up.setAlignment(QtCore.Qt.AlignCenter)
        self.label_red_up.setObjectName("label_red_up")
        self.gridLayout_3.addWidget(self.label_red_up, 0, 2, 1, 1)
        self.pushButton_red_left = QtWidgets.QPushButton(self.group_red)
        self.pushButton_red_left.setStyleSheet("")
        self.pushButton_red_left.setObjectName("pushButton_red_left")
        self.gridLayout_3.addWidget(self.pushButton_red_left, 2, 1, 1, 1)
        self.pushButton_red_right = QtWidgets.QPushButton(self.group_red)
        self.pushButton_red_right.setStyleSheet("")
        self.pushButton_red_right.setObjectName("pushButton_red_right")
        self.gridLayout_3.addWidget(self.pushButton_red_right, 2, 3, 1, 1)
        self.verticalLayout_4.addWidget(self.group_red)
        self.group_blue = QtWidgets.QGroupBox(self.tab_rgb)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.group_blue.setFont(font)
        self.group_blue.setObjectName("group_blue")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.group_blue)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label_blue_up = QtWidgets.QLabel(self.group_blue)
        self.label_blue_up.setStyleSheet("QLabel { color :blue; }")
        self.label_blue_up.setAlignment(QtCore.Qt.AlignCenter)
        self.label_blue_up.setObjectName("label_blue_up")
        self.gridLayout_4.addWidget(self.label_blue_up, 0, 2, 1, 1)
        self.pushButton_blue_up = QtWidgets.QPushButton(self.group_blue)
        self.pushButton_blue_up.setStyleSheet("")
        self.pushButton_blue_up.setObjectName("pushButton_blue_up")
        self.gridLayout_4.addWidget(self.pushButton_blue_up, 1, 2, 1, 1)
        self.label_blue_left = QtWidgets.QLabel(self.group_blue)
        self.label_blue_left.setStyleSheet("QLabel { color :blue; }")
        self.label_blue_left.setAlignment(QtCore.Qt.AlignCenter)
        self.label_blue_left.setObjectName("label_blue_left")
        self.gridLayout_4.addWidget(self.label_blue_left, 2, 0, 1, 1)
        self.pushButton_blue_left = QtWidgets.QPushButton(self.group_blue)
        self.pushButton_blue_left.setStyleSheet("")
        self.pushButton_blue_left.setObjectName("pushButton_blue_left")
        self.gridLayout_4.addWidget(self.pushButton_blue_left, 2, 1, 1, 1)
        self.pushButton_blue_reset = QtWidgets.QPushButton(self.group_blue)
        self.pushButton_blue_reset.setObjectName("pushButton_blue_reset")
        self.gridLayout_4.addWidget(self.pushButton_blue_reset, 2, 2, 1, 1)
        self.pushButton_blue_right = QtWidgets.QPushButton(self.group_blue)
        self.pushButton_blue_right.setStyleSheet("")
        self.pushButton_blue_right.setObjectName("pushButton_blue_right")
        self.gridLayout_4.addWidget(self.pushButton_blue_right, 2, 3, 1, 1)
        self.label_blue_right = QtWidgets.QLabel(self.group_blue)
        self.label_blue_right.setStyleSheet("QLabel { color :blue; }")
        self.label_blue_right.setAlignment(QtCore.Qt.AlignCenter)
        self.label_blue_right.setObjectName("label_blue_right")
        self.gridLayout_4.addWidget(self.label_blue_right, 2, 4, 1, 1)
        self.pushButton_blue_down = QtWidgets.QPushButton(self.group_blue)
        self.pushButton_blue_down.setStyleSheet("")
        self.pushButton_blue_down.setObjectName("pushButton_blue_down")
        self.gridLayout_4.addWidget(self.pushButton_blue_down, 3, 2, 1, 1)
        self.label_blue_down = QtWidgets.QLabel(self.group_blue)
        self.label_blue_down.setStyleSheet("QLabel { color :blue; }")
        self.label_blue_down.setAlignment(QtCore.Qt.AlignCenter)
        self.label_blue_down.setObjectName("label_blue_down")
        self.gridLayout_4.addWidget(self.label_blue_down, 4, 2, 1, 1)
        self.verticalLayout_4.addWidget(self.group_blue)
        self.verticalLayout_4.setStretch(0, 8)
        self.verticalLayout_4.setStretch(1, 1)
        self.verticalLayout_4.setStretch(2, 8)
        self.verticalLayout_4.setStretch(3, 8)
        self.tabWidget_postproc_control.addTab(self.tab_rgb, "")
        self.gridLayout.addWidget(self.tabWidget_postproc_control, 0, 2, 1, 2)
        self.FrameViewer = QtWidgets.QLabel(postproc_editor)
        self.FrameViewer.setText("")
        self.FrameViewer.setAlignment(QtCore.Qt.AlignCenter)
        self.FrameViewer.setObjectName("FrameViewer")
        self.gridLayout.addWidget(self.FrameViewer, 0, 1, 1, 1)
        self.label_zoom = QtWidgets.QLabel(postproc_editor)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_zoom.setFont(font)
        self.label_zoom.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_zoom.setObjectName("label_zoom")
        self.gridLayout.addWidget(self.label_zoom, 2, 1, 1, 1)
        self.label_message = QtWidgets.QLabel(postproc_editor)
        self.label_message.setObjectName("label_message")
        self.gridLayout.addWidget(self.label_message, 2, 0, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(postproc_editor)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 2, 2, 1, 1)
        self.gridLayout.setColumnStretch(0, 4)
        self.gridLayout.setColumnStretch(1, 1)

        self.retranslateUi(postproc_editor)
        self.tabWidget_postproc_control.setCurrentIndex(0)
        self.checkBox_automatic.toggled['bool'].connect(self.group_red.setDisabled)
        self.checkBox_automatic.toggled['bool'].connect(self.group_blue.setDisabled)
        QtCore.QMetaObject.connectSlotsByName(postproc_editor)

    def retranslateUi(self, postproc_editor):
        _translate = QtCore.QCoreApplication.translate
        postproc_editor.setWindowTitle(_translate("postproc_editor", "Frame"))
        self.pushButton_add_layer.setToolTip(_translate("postproc_editor", "Add a sharpening layer (up to four)"))
        self.pushButton_add_layer.setText(_translate("postproc_editor", "Add correction layer"))
        self.tabWidget_postproc_control.setTabText(self.tabWidget_postproc_control.indexOf(self.tab_sharpening), _translate("postproc_editor", "Wavelets"))
        self.group_general.setTitle(_translate("postproc_editor", "RGB Alignment parameters for the version selected"))
        self.checkBox_automatic.setToolTip(_translate("postproc_editor", "Perform RGB Alignment in fully automatic mode.\n"
"This option is available in batch mode as well."))
        self.checkBox_automatic.setText(_translate("postproc_editor", "Automatic"))
        self.fgw_label_parameter.setToolTip(_translate("postproc_editor", "In the presence of noise in the image, shift detection between frames can be misled by spurious local minima.\n"
"Gaussian blur can help finding the global optimum. Try higher values for noisy images."))
        self.fgw_label_parameter.setText(_translate("postproc_editor", "Noise level"))
        self.fgw_label_display.setText(_translate("postproc_editor", "TextLabel"))
        self.label_resolution.setText(_translate("postproc_editor", "Resolution"))
        self.comboBox_resolution.setToolTip(_translate("postproc_editor", "Choose (sub-)pixel resolution for RGB alignment. Please note\n"
"that RAM usage and execution time increase quadratically."))
        self.fgw_slider_value.setToolTip(_translate("postproc_editor", "Smoothen the image before computing the channel shifts.\n"
"In the presence of noise this results in better accuracy."))
        self.group_red.setToolTip(_translate("postproc_editor", "The current red channel shift is shown.\n"
"Use push buttons for manual adjustments."))
        self.group_red.setTitle(_translate("postproc_editor", "Red channel status / manual adjustments"))
        self.pushButton_red_up.setToolTip(_translate("postproc_editor", "Shift the red channel upwards."))
        self.pushButton_red_up.setText(_translate("postproc_editor", "Up"))
        self.label_red_left.setText(_translate("postproc_editor", "0.25"))
        self.pushButton_red_down.setToolTip(_translate("postproc_editor", "Shift the red channel down."))
        self.pushButton_red_down.setText(_translate("postproc_editor", "Down"))
        self.label_red_down.setText(_translate("postproc_editor", "0.25"))
        self.pushButton_red_reset.setToolTip(_translate("postproc_editor", "Reset the red channel shift to zero."))
        self.pushButton_red_reset.setText(_translate("postproc_editor", "Reset"))
        self.label_red_right.setText(_translate("postproc_editor", "0.25"))
        self.label_red_up.setText(_translate("postproc_editor", "0.25"))
        self.pushButton_red_left.setToolTip(_translate("postproc_editor", "Shift the red channel left."))
        self.pushButton_red_left.setText(_translate("postproc_editor", "Left"))
        self.pushButton_red_right.setToolTip(_translate("postproc_editor", "Shift the red channel right."))
        self.pushButton_red_right.setText(_translate("postproc_editor", "Right"))
        self.group_blue.setToolTip(_translate("postproc_editor", "The current blue channel shift is shown.\n"
"Use push buttons for manual adjustments."))
        self.group_blue.setTitle(_translate("postproc_editor", "Blue channel status / manual adjustments"))
        self.label_blue_up.setText(_translate("postproc_editor", "0.25"))
        self.pushButton_blue_up.setToolTip(_translate("postproc_editor", "Shift the blue channel up."))
        self.pushButton_blue_up.setText(_translate("postproc_editor", "Up"))
        self.label_blue_left.setText(_translate("postproc_editor", "0.25"))
        self.pushButton_blue_left.setToolTip(_translate("postproc_editor", "Shift the blue channel left."))
        self.pushButton_blue_left.setText(_translate("postproc_editor", "Left"))
        self.pushButton_blue_reset.setToolTip(_translate("postproc_editor", "Reset the blue channel shift to zero."))
        self.pushButton_blue_reset.setText(_translate("postproc_editor", "Reset"))
        self.pushButton_blue_right.setToolTip(_translate("postproc_editor", "Shift the blue channel right."))
        self.pushButton_blue_right.setText(_translate("postproc_editor", "Right"))
        self.label_blue_right.setText(_translate("postproc_editor", "0.25"))
        self.pushButton_blue_down.setToolTip(_translate("postproc_editor", "Shift the blue channel down."))
        self.pushButton_blue_down.setText(_translate("postproc_editor", "Down"))
        self.label_blue_down.setText(_translate("postproc_editor", "0.25"))
        self.tabWidget_postproc_control.setTabText(self.tabWidget_postproc_control.indexOf(self.tab_rgb), _translate("postproc_editor", "RGB Alignment"))
        self.label_zoom.setText(_translate("postproc_editor", "100%"))
        self.label_message.setText(_translate("postproc_editor", "message"))
        self.buttonBox.setToolTip(_translate("postproc_editor", "Press \"OK\" to save the selected version and exit, or \"Cancel\" to discard changes"))
