# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'sharpening_layer_widget.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_sharpening_layer_widget(object):
    def setupUi(self, sharpening_layer_widget):
        sharpening_layer_widget.setObjectName("sharpening_layer_widget")
        sharpening_layer_widget.resize(311, 227)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(sharpening_layer_widget.sizePolicy().hasHeightForWidth())
        sharpening_layer_widget.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(sharpening_layer_widget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox_layer = QtWidgets.QGroupBox(sharpening_layer_widget)
        self.groupBox_layer.setObjectName("groupBox_layer")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_layer)
        self.gridLayout.setHorizontalSpacing(12)
        self.gridLayout.setObjectName("gridLayout")
        self.label_amount = QtWidgets.QLabel(self.groupBox_layer)
        self.label_amount.setObjectName("label_amount")
        self.gridLayout.addWidget(self.label_amount, 1, 1, 1, 1)
        self.checkBox_luminance = QtWidgets.QCheckBox(self.groupBox_layer)
        self.checkBox_luminance.setObjectName("checkBox_luminance")
        self.gridLayout.addWidget(self.checkBox_luminance, 7, 1, 1, 2)
        self.label_radius = QtWidgets.QLabel(self.groupBox_layer)
        self.label_radius.setObjectName("label_radius")
        self.gridLayout.addWidget(self.label_radius, 0, 1, 1, 1)
        self.pushButton_remove = QtWidgets.QPushButton(self.groupBox_layer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButton_remove.sizePolicy().hasHeightForWidth())
        self.pushButton_remove.setSizePolicy(sizePolicy)
        self.pushButton_remove.setMinimumSize(QtCore.QSize(60, 0))
        self.pushButton_remove.setMaximumSize(QtCore.QSize(60, 16777215))
        self.pushButton_remove.setObjectName("pushButton_remove")
        self.gridLayout.addWidget(self.pushButton_remove, 7, 3, 1, 1)
        self.lineEdit_radius = QtWidgets.QLineEdit(self.groupBox_layer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_radius.sizePolicy().hasHeightForWidth())
        self.lineEdit_radius.setSizePolicy(sizePolicy)
        self.lineEdit_radius.setMinimumSize(QtCore.QSize(60, 0))
        self.lineEdit_radius.setMaximumSize(QtCore.QSize(60, 16777215))
        self.lineEdit_radius.setObjectName("lineEdit_radius")
        self.gridLayout.addWidget(self.lineEdit_radius, 0, 3, 1, 1)
        self.horizontalSlider_bi_fraction = QtWidgets.QSlider(self.groupBox_layer)
        self.horizontalSlider_bi_fraction.setMaximum(100)
        self.horizontalSlider_bi_fraction.setPageStep(5)
        self.horizontalSlider_bi_fraction.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSlider_bi_fraction.setObjectName("horizontalSlider_bi_fraction")
        self.gridLayout.addWidget(self.horizontalSlider_bi_fraction, 3, 2, 1, 1)
        self.bilateral_fraction_label = QtWidgets.QLabel(self.groupBox_layer)
        self.bilateral_fraction_label.setObjectName("bilateral_fraction_label")
        self.gridLayout.addWidget(self.bilateral_fraction_label, 3, 1, 1, 1)
        self.lineEdit_bi_fraction = QtWidgets.QLineEdit(self.groupBox_layer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_bi_fraction.sizePolicy().hasHeightForWidth())
        self.lineEdit_bi_fraction.setSizePolicy(sizePolicy)
        self.lineEdit_bi_fraction.setMinimumSize(QtCore.QSize(60, 0))
        self.lineEdit_bi_fraction.setMaximumSize(QtCore.QSize(60, 16777215))
        self.lineEdit_bi_fraction.setObjectName("lineEdit_bi_fraction")
        self.gridLayout.addWidget(self.lineEdit_bi_fraction, 3, 3, 1, 1)
        self.lineEdit_amount = QtWidgets.QLineEdit(self.groupBox_layer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_amount.sizePolicy().hasHeightForWidth())
        self.lineEdit_amount.setSizePolicy(sizePolicy)
        self.lineEdit_amount.setMinimumSize(QtCore.QSize(60, 0))
        self.lineEdit_amount.setMaximumSize(QtCore.QSize(60, 16777215))
        self.lineEdit_amount.setObjectName("lineEdit_amount")
        self.gridLayout.addWidget(self.lineEdit_amount, 1, 3, 1, 1)
        self.horizontalSlider_amount = QtWidgets.QSlider(self.groupBox_layer)
        self.horizontalSlider_amount.setMaximum(100)
        self.horizontalSlider_amount.setPageStep(1)
        self.horizontalSlider_amount.setProperty("value", 3)
        self.horizontalSlider_amount.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSlider_amount.setObjectName("horizontalSlider_amount")
        self.gridLayout.addWidget(self.horizontalSlider_amount, 1, 2, 1, 1)
        self.horizontalSlider_radius = QtWidgets.QSlider(self.groupBox_layer)
        self.horizontalSlider_radius.setMinimum(1)
        self.horizontalSlider_radius.setMaximum(100)
        self.horizontalSlider_radius.setPageStep(1)
        self.horizontalSlider_radius.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSlider_radius.setObjectName("horizontalSlider_radius")
        self.gridLayout.addWidget(self.horizontalSlider_radius, 0, 2, 1, 1)
        self.label_bi_range = QtWidgets.QLabel(self.groupBox_layer)
        self.label_bi_range.setObjectName("label_bi_range")
        self.gridLayout.addWidget(self.label_bi_range, 4, 1, 1, 1)
        self.horizontalSlider_bi_range = QtWidgets.QSlider(self.groupBox_layer)
        self.horizontalSlider_bi_range.setMaximum(200)
        self.horizontalSlider_bi_range.setPageStep(2)
        self.horizontalSlider_bi_range.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSlider_bi_range.setObjectName("horizontalSlider_bi_range")
        self.gridLayout.addWidget(self.horizontalSlider_bi_range, 4, 2, 1, 1)
        self.lineEdit_bi_range = QtWidgets.QLineEdit(self.groupBox_layer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_bi_range.sizePolicy().hasHeightForWidth())
        self.lineEdit_bi_range.setSizePolicy(sizePolicy)
        self.lineEdit_bi_range.setMaximumSize(QtCore.QSize(60, 16777215))
        self.lineEdit_bi_range.setObjectName("lineEdit_bi_range")
        self.gridLayout.addWidget(self.lineEdit_bi_range, 4, 3, 1, 1)
        self.label_denoise = QtWidgets.QLabel(self.groupBox_layer)
        self.label_denoise.setObjectName("label_denoise")
        self.gridLayout.addWidget(self.label_denoise, 5, 1, 1, 1)
        self.horizontalSlider_denoise = QtWidgets.QSlider(self.groupBox_layer)
        self.horizontalSlider_denoise.setMaximum(100)
        self.horizontalSlider_denoise.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSlider_denoise.setObjectName("horizontalSlider_denoise")
        self.gridLayout.addWidget(self.horizontalSlider_denoise, 5, 2, 1, 1)
        self.lineEdit_denoise = QtWidgets.QLineEdit(self.groupBox_layer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_denoise.sizePolicy().hasHeightForWidth())
        self.lineEdit_denoise.setSizePolicy(sizePolicy)
        self.lineEdit_denoise.setMaximumSize(QtCore.QSize(60, 16777215))
        self.lineEdit_denoise.setObjectName("lineEdit_denoise")
        self.gridLayout.addWidget(self.lineEdit_denoise, 5, 3, 1, 1)
        self.verticalLayout.addWidget(self.groupBox_layer)

        self.retranslateUi(sharpening_layer_widget)
        QtCore.QMetaObject.connectSlotsByName(sharpening_layer_widget)

    def retranslateUi(self, sharpening_layer_widget):
        _translate = QtCore.QCoreApplication.translate
        sharpening_layer_widget.setWindowTitle(_translate("sharpening_layer_widget", "Form"))
        self.groupBox_layer.setTitle(_translate("sharpening_layer_widget", "Layer 1"))
        self.label_amount.setText(_translate("sharpening_layer_widget", "Amount"))
        self.checkBox_luminance.setToolTip(_translate("sharpening_layer_widget", "Apply sharpening only to luminance channel"))
        self.checkBox_luminance.setText(_translate("sharpening_layer_widget", "Luminance channel only"))
        self.label_radius.setText(_translate("sharpening_layer_widget", "Radius"))
        self.pushButton_remove.setToolTip(_translate("sharpening_layer_widget", "Remove this layer"))
        self.pushButton_remove.setText(_translate("sharpening_layer_widget", "Remove"))
        self.lineEdit_radius.setToolTip(_translate("sharpening_layer_widget", "Radius of Gauss filter"))
        self.horizontalSlider_bi_fraction.setToolTip(_translate("sharpening_layer_widget", "Set the fraction of the bilateral filter component (vs. Gaussian)"))
        self.bilateral_fraction_label.setText(_translate("sharpening_layer_widget", "<html><head/><body><p>Bilateral<br/>Fraction</p></body></html>"))
        self.lineEdit_bi_fraction.setToolTip(_translate("sharpening_layer_widget", "Fraction of bilateral filter component"))
        self.lineEdit_amount.setToolTip(_translate("sharpening_layer_widget", "Relative effect of this layer"))
        self.horizontalSlider_amount.setToolTip(_translate("sharpening_layer_widget", "Increase / decrease the effect of this layer"))
        self.horizontalSlider_radius.setToolTip(_translate("sharpening_layer_widget", "Change the radius of Gauss filter for this layer"))
        self.label_bi_range.setText(_translate("sharpening_layer_widget", "<html><head/><body><p>Bilateral<br/>Range</p></body></html>"))
        self.horizontalSlider_bi_range.setToolTip(_translate("sharpening_layer_widget", "Set the brigthness range parameter of the bilateral filter"))
        self.lineEdit_bi_range.setToolTip(_translate("sharpening_layer_widget", "Range parameter of bilateral filter"))
        self.label_denoise.setText(_translate("sharpening_layer_widget", "Denoise"))
        self.horizontalSlider_denoise.setToolTip(_translate("sharpening_layer_widget", "Set the strength of the denoise filter (Gaussian) for this layer"))
        self.lineEdit_denoise.setToolTip(_translate("sharpening_layer_widget", "Strength of the denoise filter (Gaussian) for this layer"))
