# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'alignment_point_editor_gui.ui'
#
# Created by: PyQt5 UI code generator 5.14.2
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_alignment_point_editor(object):
    def setupUi(self, alignment_point_editor):
        alignment_point_editor.setObjectName("alignment_point_editor")
        alignment_point_editor.resize(941, 629)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(alignment_point_editor.sizePolicy().hasHeightForWidth())
        alignment_point_editor.setSizePolicy(sizePolicy)
        alignment_point_editor.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        alignment_point_editor.setFrameShape(QtWidgets.QFrame.Panel)
        alignment_point_editor.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(alignment_point_editor)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.aphbw_label_display = QtWidgets.QLabel(alignment_point_editor)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.aphbw_label_display.sizePolicy().hasHeightForWidth())
        self.aphbw_label_display.setSizePolicy(sizePolicy)
        self.aphbw_label_display.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.aphbw_label_display.setObjectName("aphbw_label_display")
        self.verticalLayout_3.addWidget(self.aphbw_label_display)
        self.aphbw_slider_value = QtWidgets.QSlider(alignment_point_editor)
        self.aphbw_slider_value.setMinimum(20)
        self.aphbw_slider_value.setMaximum(140)
        self.aphbw_slider_value.setSingleStep(4)
        self.aphbw_slider_value.setPageStep(4)
        self.aphbw_slider_value.setProperty("value", 40)
        self.aphbw_slider_value.setOrientation(QtCore.Qt.Vertical)
        self.aphbw_slider_value.setObjectName("aphbw_slider_value")
        self.verticalLayout_3.addWidget(self.aphbw_slider_value)
        self.aphbw_label_parameter = QtWidgets.QLabel(alignment_point_editor)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.aphbw_label_parameter.sizePolicy().hasHeightForWidth())
        self.aphbw_label_parameter.setSizePolicy(sizePolicy)
        self.aphbw_label_parameter.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.aphbw_label_parameter.setObjectName("aphbw_label_parameter")
        self.verticalLayout_3.addWidget(self.aphbw_label_parameter)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem1)
        self.apst_label_display = QtWidgets.QLabel(alignment_point_editor)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.apst_label_display.sizePolicy().hasHeightForWidth())
        self.apst_label_display.setSizePolicy(sizePolicy)
        self.apst_label_display.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.apst_label_display.setObjectName("apst_label_display")
        self.verticalLayout_3.addWidget(self.apst_label_display)
        self.apst_slider_value = QtWidgets.QSlider(alignment_point_editor)
        self.apst_slider_value.setMinimum(1)
        self.apst_slider_value.setMaximum(30)
        self.apst_slider_value.setPageStep(5)
        self.apst_slider_value.setProperty("value", 5)
        self.apst_slider_value.setOrientation(QtCore.Qt.Vertical)
        self.apst_slider_value.setObjectName("apst_slider_value")
        self.verticalLayout_3.addWidget(self.apst_slider_value)
        self.apst_label_parameter = QtWidgets.QLabel(alignment_point_editor)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.apst_label_parameter.sizePolicy().hasHeightForWidth())
        self.apst_label_parameter.setSizePolicy(sizePolicy)
        self.apst_label_parameter.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.apst_label_parameter.setObjectName("apst_label_parameter")
        self.verticalLayout_3.addWidget(self.apst_label_parameter)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem2)
        self.apbt_label_display = QtWidgets.QLabel(alignment_point_editor)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.apbt_label_display.sizePolicy().hasHeightForWidth())
        self.apbt_label_display.setSizePolicy(sizePolicy)
        self.apbt_label_display.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.apbt_label_display.setObjectName("apbt_label_display")
        self.verticalLayout_3.addWidget(self.apbt_label_display)
        self.apbt_slider_value = QtWidgets.QSlider(alignment_point_editor)
        self.apbt_slider_value.setMinimum(2)
        self.apbt_slider_value.setMaximum(50)
        self.apbt_slider_value.setSingleStep(2)
        self.apbt_slider_value.setPageStep(2)
        self.apbt_slider_value.setProperty("value", 10)
        self.apbt_slider_value.setOrientation(QtCore.Qt.Vertical)
        self.apbt_slider_value.setObjectName("apbt_slider_value")
        self.verticalLayout_3.addWidget(self.apbt_slider_value)
        self.apbt_label_parameter = QtWidgets.QLabel(alignment_point_editor)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.apbt_label_parameter.sizePolicy().hasHeightForWidth())
        self.apbt_label_parameter.setSizePolicy(sizePolicy)
        self.apbt_label_parameter.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.apbt_label_parameter.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.apbt_label_parameter.setObjectName("apbt_label_parameter")
        self.verticalLayout_3.addWidget(self.apbt_label_parameter)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem3)
        self.restore_standard_values = QtWidgets.QPushButton(alignment_point_editor)
        self.restore_standard_values.setObjectName("restore_standard_values")
        self.verticalLayout_3.addWidget(self.restore_standard_values)
        self.horizontalLayout_2.addLayout(self.verticalLayout_3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btnApGrid = QtWidgets.QPushButton(alignment_point_editor)
        self.btnApGrid.setObjectName("btnApGrid")
        self.horizontalLayout.addWidget(self.btnApGrid)
        self.btnUndo = QtWidgets.QPushButton(alignment_point_editor)
        self.btnUndo.setObjectName("btnUndo")
        self.horizontalLayout.addWidget(self.btnUndo)
        self.btnRedo = QtWidgets.QPushButton(alignment_point_editor)
        self.btnRedo.setObjectName("btnRedo")
        self.horizontalLayout.addWidget(self.btnRedo)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.label_message = QtWidgets.QLabel(alignment_point_editor)
        self.label_message.setObjectName("label_message")
        self.horizontalLayout.addWidget(self.label_message)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem5)
        self.buttonBox = QtWidgets.QDialogButtonBox(alignment_point_editor)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.horizontalLayout.addWidget(self.buttonBox)
        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.retranslateUi(alignment_point_editor)
        self.aphbw_slider_value.valueChanged['int'].connect(self.aphbw_label_display.setNum)
        self.apbt_slider_value.valueChanged['int'].connect(self.apbt_label_display.setNum)
        QtCore.QMetaObject.connectSlotsByName(alignment_point_editor)

    def retranslateUi(self, alignment_point_editor):
        _translate = QtCore.QCoreApplication.translate
        alignment_point_editor.setWindowTitle(_translate("alignment_point_editor", "Frame"))
        self.aphbw_label_display.setText(_translate("alignment_point_editor", "TextLabel"))
        self.aphbw_label_parameter.setToolTip(_translate("alignment_point_editor", "Size of the quadratic box around an alignment point used to measure the local shift.\n"
"Smaller values result in better resolution of locality, but shift detection can fail if local contrast is low."))
        self.aphbw_label_parameter.setText(_translate("alignment_point_editor", "Alignment box\n"
"width (pixels)"))
        self.apst_label_display.setText(_translate("alignment_point_editor", "TextLabel"))
        self.apst_label_parameter.setToolTip(_translate("alignment_point_editor", "In the automatic construction of alignment point grids, points are discarded if there is too little local structure.\n"
"Higher values result in more points to be discarded. If the value is too low, unreliable alignment points are included."))
        self.apst_label_parameter.setText(_translate("alignment_point_editor", "Minimum\n"
"structure"))
        self.apbt_label_display.setText(_translate("alignment_point_editor", "TextLabel"))
        self.apbt_label_parameter.setToolTip(_translate("alignment_point_editor", "In the automatic construction of alignment point grids, points are discarded if no pixels in the surrounding box are bright enough.\n"
"Higher values result in more points to be discarded. If the value is too low, unreliable alignment points are included."))
        self.apbt_label_parameter.setText(_translate("alignment_point_editor", "Minimum\n"
"brightness"))
        self.restore_standard_values.setToolTip(_translate("alignment_point_editor", "Reset parameters to original values. In most cases they should give satisfactory results."))
        self.restore_standard_values.setText(_translate("alignment_point_editor", "Restore\n"
"standard\n"
"values"))
        self.btnApGrid.setToolTip(_translate("alignment_point_editor", "Create an alignment point grid automatically."))
        self.btnApGrid.setText(_translate("alignment_point_editor", "Create AP Grid"))
        self.btnUndo.setToolTip(_translate("alignment_point_editor", "Undo the last step."))
        self.btnUndo.setText(_translate("alignment_point_editor", "Undo"))
        self.btnRedo.setToolTip(_translate("alignment_point_editor", "Redo the last step."))
        self.btnRedo.setText(_translate("alignment_point_editor", "Redo"))
        self.label_message.setText(_translate("alignment_point_editor", "Create an AP grid, or create / modify alignment points manually (with the \'ctrl\' button pressed). Confirm by pressing \'OK\'."))
        self.buttonBox.setToolTip(_translate("alignment_point_editor", "Confirm the current AP selection."))
